<?php
// webtrees: Web based Family History software
// Copyright (C) 2014 webtrees development team.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

/*
*     Class PlacesDoc V-2-7-2-1
*      pour Place_doc V-2-7-2-1
*     Andr DUMAS DEC 2015
*     pour Webtrees v.1.7.2
*     http://www.adumas.1s.fr/wt
*      http://www.adumas.1s.fr/download/
*/

namespace Andre\WebtreesAddOn\PlaceDoc;

use Composer\Autoload\ClassLoader;
use Fisharebest\Webtrees\Controller\BaseController;
use Fisharebest\Webtrees\Controller\PageController;
use Fisharebest\Webtrees\Filter;
use Fisharebest\Webtrees\I18N;
use Fisharebest\Webtrees\Tree;
use Fisharebest\Webtrees\Log;
use Fisharebest\Webtrees\GedcomRecord;
use Fisharebest\Webtrees\FlashMessages;
use Fisharebest\Webtrees\Individual;
use Fisharebest\Webtrees\Menu;
use Fisharebest\Webtrees\Media;
use Fisharebest\Webtrees\Auth;
use Fisharebest\Webtrees\Database;
use Fisharebest\Webtrees\Functions;




/**
 * Class PlacesDoc
 */
 
class PlacesDocClass extends PlaceDocModule  {
	private $LG_codeINSEE =6;  //Longueur du code des villes en France + 1
	private $places = array();  //lieux associs  des vnements pour cet Id
	private $nb;  // nombre de champs du format des lieux dans HEAD
   
	/**
	 * PlacesDoc Constructor
	 *
	 * @param string $name the name of the TreeView objects instance
	 */
	function __construct() {
		//$this->places = $this->getPlacesArray();
		$this->nb = $this->get_plac_label();
	}


private function	getPlacesArray(Individual $individual) {
		global $SHOW_LEVEL2_NOTES, $NAV_NOTES, $controller;
		global $TEXT_DIRECTION;
	global $SEARCH_SPIDER;
	global $THUMBNAIL_WIDTH;
	global $GEDCOM, $HIDE_GEDCOM_ERRORS;	
	
	
		$places = array();
		
		
foreach ($individual->getFacts() as $fact) {
			if (!($fact->getPlace()->isEmpty())) {
				$places[] = $fact->getPlace()->getGedcomName();
			}
	};	
	return array_unique($places);
	}

protected function get_plac_label() {
		global $GEDCOM, $WT_TREE;
		$ged_id=$WT_TREE->getTreeId();
/* Certains recors HEAD dans ##other ne sont pas renseigns
*  amors on prend des valeurs par defaut pour PLAC FORM conforme au 
*  schema adopt pour les lieux Franais
*/		
		$record = GedcomRecord::getInstance('HEAD', $WT_TREE);
		if (($fact = $record->getFirstFact('PLAC')) != null) {
		$PLAC = "\n1 PLAC\n2 FORM " . $fact->getAttribute('FORM');	
		$HEAD_PLAC_FORM = $fact->getAttribute('FORM');  //V 1.7.1
		$HEAD_PLAC_FORM = substr($HEAD_PLAC_FORM, 7);
		} else {
		 $HEAD_PLAC_FORM = I18N::translate('City, County, State/Province, Country'); echo '2';
		 }
		
	return substr_count($HEAD_PLAC_FORM,',');
}	


//	--------------------------------
public function ViewPldSites(Individual $individual) {
    echo($this->genDiffSites($individual));
}	
	
private function genDiffSites(Individual $individual) {
	
	$sites = array();
	$pls = array();
	$nb_champs_format_lieux = $this->nb;
	
	$places = $this->getPlacesArray($individual);
	
	foreach ($places as $lieu) {
	     if (!empty($lieu)) { $ple = explode(",", $lieu);
								if (count($ple) != $nb_champs_format_lieux + 1) {
									//Message
									FlashMessages::addMessage(
										I18N::translate('Format is not conform !')
									);
									continue;
									} else {
										$pls[] = $ple; //on ne conserve que les lieux avec le bon format
										$sites[] = array(	
															'ville'=>
														utf8_encode(substr(trim($ple[0]),0, strlen(trim($ple[0]))-$this->LG_codeINSEE)),
															'code'=>
														utf8_encode(trim(substr(trim($ple[0]), strlen(trim($ple[0]))-$this->LG_codeINSEE,strlen(trim($ple[0]))))),
															'pays'=>
															utf8_encode(trim($ple[$nb_champs_format_lieux]))
														);
										}	
														}
	}
	
		return json_encode($sites);
}	

public function ViewPldPlaces() {
    
}



public function genHtmlVilles($individual, $pays) {
	global $WT_TREE;
	$sites = array();
	$lg_codeINSEE = 5+1;
	$pls = array();
	$nb_champs_format_lieux = $this->nb;
	$places = $this->getPlacesArray($individual);  
	$html = '';
	$coord  = array();
	
	if ( (trim($pays) != 'FRANCE') && (trim($pays) != 'FRA') && (trim($pays) != 'FR') ) {$lg_codeINSEE = 0; }
	
	
	/*    Si le dernier champ est PAYS Country
	*   Si $pays === FRANCE FRA FR  et nb_champs_format = 4
	*   et si premier champ est 'Ville' NON numeric
	*   Alors  deuxieme champ est codeZIP  (Ville, Code, Dept, Reg, PAYS)
	*   Si le premier champ est numerix alors (Code, Ville, Dept, Reg, PAYS)
	*
	*   si nbre_champs == 3 format FR "classique  WT"  (Ville Code, Dept, Reg, Pays)
	*/
	
   switch ($this->nb) {
    case '3':	   
	foreach ($places as $lieu) {
	     if (!empty($lieu)) { 
						// Chercher dans ##placelocation les lat & long du lieu
						$ple = explode(",", $lieu);
								if (count($ple) != ($nb_champs_format_lieux + 1)) {									
									FlashMessages::addMessage(
										I18N::translate('Format is not conform !')
									);
									continue;
									} else {
										$pls[] = $ple; //on ne conserve que les lieux avec le bon format
									if ($pays == utf8_encode(trim($ple[$nb_champs_format_lieux]))) {											
										$coord = $this->selectPlace($ple);
										$idv = utf8_encode(substr(trim($ple[0]),0, strlen(trim($ple[0]))-$lg_codeINSEE)).'_'.$coord[0]->pl_long.'_'.$coord[0]->pl_lati;
										
									
										//creation de la source si ncessaire et si admin
										if (Auth::isAdmin() && ($coord != null) ) {
											if ($this->existSource($idv, $WT_TREE->getTreeId()) == null) {
												$this->createSource($idv);
											}
										}
									
										$html .=  '<option value="'.	
														$idv.'">'.
														utf8_encode(substr(trim($ple[0]),0, strlen(trim($ple[0]))-$lg_codeINSEE)).
														'</option>';				
									}						
		
										}	
														}
	}
	break;
	case '4':
	foreach ($places as $lieu) {
	     if (!empty($lieu)) { 
						$ple = explode(",", $lieu);
								if (count($ple) != $nb_champs_format_lieux + 1) {									
									FlashMessages::addMessage(I18N::translate('Format is not conform !'));
									continue;
									} else {
										$pls[] = $ple; //on ne conserve que les lieux avec le bon format
										if ($pays == utf8_encode(trim($ple[$nb_champs_format_lieux]))) {	
									
									$coord = $this->selectPlace($ple);		
										//si le lieu n'existe pas dans la BD sous cette forme
										   // if (empty($coord) { continue;}
											
										//	
										
											   if (is_numeric(trim($ple[0]))) { 
												   // Code first
												    
												$idv = utf8_encode(trim($ple[1])).'_'.$coord[0]->pl_long.'_'.$coord[0]->pl_lati;
										
									//creation de la source si ncessaire et si admin
									if (Auth::isAdmin()) {
										if ($this->existSource($idv, $WT_TREE->getTreeId()) == null) {
											$this->createSource($idv);
										}
									} 
												   
												   
												$html .=  '<option value="'.	
																utf8_encode(trim($idv)).'">'.utf8_encode(trim($ple[1])).	 
																'</option>';
												} else {  	
												// Ville first
												$idv = utf8_encode(trim($ple[0])).'_'.$coord[0]->pl_long.'_'.$coord[0]->pl_lati;
									
									//creation de la source si ncessaire et si admin
									if (Auth::isAdmin()) {
										if ($this->existSource($idv, $WT_TREE->getTreeId()) == null) {
											$this->createSource($idv);
										}
									}
												$html .=  '<option value="'.	
																$idv.'">'.utf8_encode(trim($ple[0])).	 
																'</option>';	
												}										
										}	
								}
		}
			}
	break;
	case '5':
		// cas du lieu-dit  TODO
	break;
   }
   // $this->setLaSource($idv);
	return $html;
	
}


public function genHtmlPays($individual) {
	
	$lg_codeINSEE = 5+1;
	$pls = array();
	$nb_champs_format_lieux = $this->nb;
	$places = $this->getPlacesArray($individual); 
	$html = '';
	$p = '';
	
	foreach ($places as $lieu) {  
	     if (!empty($lieu)) { $ple = explode(",", $lieu);
								if (count($ple) != $nb_champs_format_lieux + 1) {
									
									continue;
									} else {
										$pls[] = $ple; //on ne conserve que les lieux avec le bon format
										
                                       $p1 =  utf8_encode(trim($ple[$nb_champs_format_lieux])); 
									if (($p1 != $p) AND (stripos($html, $p1) != true)){	
											$html .= '<option value="'.$p1.'">'.$p1.'</option>'; 
											
									}
										}	
									$p = $p1;	
														}
	}
	
		return $html;
	
}

/*
*    Determine Lat et Long de $place with $level
*    from ##placelocation
*
*/
private function getCoord( $place, $level) {
		
	$sql = "SELECT SQL_CACHE pl_long, pl_lati FROM ##placelocation WHERE pl_place LIKE '%".$place."%' AND pl_level ='".$level."'";

			$places=
				Database::prepare($sql)->execute()->fetchOne();
				
			return $places;
}

private function getCoordPlace($code)  {


//Dans la requete il ne faut pas mettre les ' autour des noms de table et de champ  !!!!

$sql = "SELECT SQL_CACHE pl_long, pl_lati FROM ##placelocation WHERE pl_place LIKE '%".$code."%';";		

			$places=
				Database::prepare($sql)->execute()->fetchAll();
				
			return $places;
			
	}	
	
	/*
	*   Dtermine prcisment les lat et lon d'un lieu
	*   en tant compte des homonymies dans des pays, rgions diffrentes pour tous les pays
	*   Pour 4 champs (Ville, dept, region, pays) 
	*/
	private function selectPlace($lieu) {
	switch ($this->nb) {	
		case '3':
	$level = 3;	
		$sql = "SELECT SQL_CACHE p1.pl_long, p1.pl_lati  FROM ##placelocation AS p1 ".
				" LEFT JOIN `##placelocation` AS p2 ON (p1.pl_parent_id = p2.pl_id) AND (p2.pl_place LIKE '%".trim($lieu[1])."%')" .
				" LEFT JOIN `##placelocation` AS p3 ON (p2.pl_parent_id = p3.pl_id) AND (p3.pl_place LIKE'%".trim($lieu[2])."%')".
				" LEFT JOIN `##placelocation` AS p4 ON (p3.pl_parent_id = p4.pl_id) AND (p4.pl_place LIKE '%".trim($lieu[3])."%')". 
				" WHERE p1.pl_place LIKE '%".$lieu[0]."%' AND p1.pl_level ='".$level."'			
			";
		break;
		case '4' :
		$level = 4;	
		$sql = "SELECT SQL_CACHE p1.pl_long, p1.pl_lati  FROM ##placelocation AS p1 ".
				" LEFT JOIN `##placelocation` AS p2 ON (p1.pl_parent_id = p2.pl_id) AND (p2.pl_place LIKE '%".trim($lieu[1])."%')" .
				" LEFT JOIN `##placelocation` AS p3 ON (p2.pl_parent_id = p3.pl_id) AND (p3.pl_place LIKE '%".trim($lieu[2])."%')" .
				" LEFT JOIN `##placelocation` AS p4 ON (p3.pl_parent_id = p4.pl_id) AND (p4.pl_place LIKE'%".trim($lieu[3])."%')".
				" LEFT JOIN `##placelocation` AS p5 ON (p4.pl_parent_id = p5.pl_id) AND (p5.pl_place LIKE '%".trim($lieu[4])."%')". 
				" WHERE p1.pl_place LIKE '%".$lieu[0]."%' AND p1.pl_level ='".$level."'			
			";
		break;
	};
	
			$place= Database::prepare($sql)->execute()->fetchAll();
			return $place;
	}
	
	

   /*
	* Si aucune source dont le titre est du type S_title existe
	* alors on la cre
	**/
		private function createSource($title) {
		global $WT_TREE;
		//ref edit-interface
		
		$type = 'SOUR';
		$xref   = $WT_TREE->getNewXref($type);
		
			$newgedrec = '0 @'.$xref.'@ SOUR';
			
	$ABBR      = 'S_'.$title;
	if ($ABBR) {
		$newgedrec .= "\n1 ABBR " . $ABBR;
	}
	$TITL = 'S_'.$title;
	if ($TITL) {
		$newgedrec .= "\n1 TITL " . $TITL;
	}
	
	
	Database::prepare(
			"INSERT INTO `##sources` (s_id, s_file, s_name, s_gedcom) 
			VALUES (:id, :gid, :name,:ged )"
		)->execute(array(
						"id"    =>  $xref,
						"gid"   =>  $WT_TREE->getTreeId() ,
						"name"  => $TITL ,
						"ged"   => $newgedrec,
						));
	
		Log::addEditLog('Create: ' . $type . ' ' . $xref);
	
	return true;	
	}
	
	/**
	* Vrifie si une source dont le titre est du type S_title existe
	* dans la table ##sources
	* getIdSource($source, $gid = $WT_TREE->getTreeId()) ligne 921
	**/
	protected function existSource($title, $gid ) {
		
		$data = Database::prepare(
			"SELECT SQL_CACHE s_id FROM `##sources` WHERE s_name = ? AND s_file = ? ")
			->execute(array(
				'S_'.$title,
				$gid,
					))->fetchOne();
		
	return $data;	
	}
	
}	
?>