<?php
/*
 * webtrees: online genealogy
 * Copyright (C) 2015 webtrees development team
 * Copyright (C) 2015 JustCarmen
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
namespace Andre\WebtreesAddOn\PlaceDoc\Template;

use Fisharebest\Webtrees\Auth;
use Fisharebest\Webtrees\Controller\PageController;
use Fisharebest\Webtrees\Filter;
use Fisharebest\Webtrees\Site;
use Fisharebest\Webtrees\Functions\FunctionsEdit;
use Fisharebest\Webtrees\Functions\FunctionsPrint;
use Fisharebest\Webtrees\I18N;
use Fisharebest\Webtrees\Tree;

use Andre\WebtreesAddOn\PlaceDoc\PlaceDocModule;

use Andre\WebtreesAddOn\PlaceDoc\myUpdateFunctions;

class AdminTemplate extends PlaceDocModule {

	protected function pageContent() {
		$controller = new PageController;
		return
			$this->pageHeader($controller) .
			$this->pageBody($controller);
	}

	private function pageHeader(PageController $controller) {
		$fjs = WT_MODULES_DIR . $this->getName() . '/js/my.js';
		$msgFieldEmpty = I18N::translate('Fields with a * can not be empty !! ');
	
	
		$controller
			->restrictAccess(Auth::isAdmin())
			->setPageTitle(I18N::translate($this->getTitle()))
			->pageHeader()
			->addInlineJavascript('
							function include_css(css_file) {
								var html_doc = document.getElementsByTagName("head")[0];
								var css = document.createElement("link");
								css.setAttribute("rel", "stylesheet");
								css.setAttribute("type", "text/css");
								css.setAttribute("href", css_file);
								html_doc.appendChild(css);
							}
							include_css("' . WT_MODULES_DIR . $this->getName() . '/' . 'css/' . 'my.css'.'");
							
						')
			->addInlineJavascript('var modName =\'' . $this->getName() .'\'
									var msgFieldEmpty = \'' .$msgFieldEmpty . '\' ')
			->addExternalJavascript($fjs);
						
	}

	private function pageBody(PageController $controller) {
	
	echo $this->getBandeau($controller);
if (Filter::postBool('save')) {
			
				$gallery_url = Filter::post('NEW_PLD_GALLERY_URL');
				$this->setSetting('PLD_SETTING_GALLERY_URL', $gallery_url);
				$delcampe_url = Filter::post('NEW_PLD_DELCAMPE_URL');
				$this->setSetting('PLD_SETTING_DELCAMPE_URL', $delcampe_url);
				$cassini_url = Filter::post('NEW_PLD_CASSINI_URL');
				$this->setSetting('PLD_SETTING_CASSINI_URL', $cassini_url);
				$welcome_source = Filter::post('NEW_PLD_WELCOME_SOURCE');
				if ($welcome_source == '') {$welcome_source = 'S_Paris_E2.4167_N48.8167';}
				$this->setSetting('PLD_SETTING_WELCOME_SOURCE', $welcome_source);
				
			
			
			
}			
			//set module settings
			
			
		// get module settings (options are coming from function options)
		//$PLD_SETTINGS = unserialize($this->getSetting('PLD_SETTINGS'));

		// inline javascript
		$controller->addInlineJavascript('
			function include_css(css_file) {
				var html_doc = document.getElementsByTagName("head")[0];
				var css = document.createElement("link");
				css.setAttribute("rel", "stylesheet");
				css.setAttribute("type", "text/css");
				css.setAttribute("href", css_file);
				html_doc.appendChild(css);
			}
			include_css("' . WT_MODULES_DIR . $this->getName() . '/' . 'themes' . 'style.css");
			var pastefield; function paste_id(value) { pastefield.value=value; } // For the \'find indi\' link

			
$(document).ready(function(){
		 $("input[type=submit]").click(function(e){
				//e.preventDefault(); // on annule la fonction par défaut du bouton envoyer
				
				verification();   
			});
	});		
	
function verification(){
				var $urlg = $("#NEW_PLD_GALLERY_URL"); 
				var $urlc = $("#NEW_PLD_CASSINI_URL");
				
				if ($urlg.val().split("://").length == 1) {
					//on ajoute http://
					var $newurl = "http://" + $("#NEW_PLD_GALLERY_URL").val();
					$urlg.val( $newurl );
					$("a#test_url_gallery").attr("href", $newurl);
				};
				if ($urlc.val().split("://").length == 1) {
					//on ajoute http://
					var $newurl = "http://" + $("#NEW_PLD_CASSINI_URL").val();
					$urlc.val( $newurl );
					$("a#test_url_cassini").attr("href", $newurl);
				};
			};		
			
			
			
		');

		// Admin page content
		global $WT_TREE;
		$html = '
			<div id="places-documentation-config">
				<div id="error"></div>
						<br>
			<form method="post" name="configform" action="' . $this->getConfigLink() . '">
				<input type="hidden" id="save" name="save" value="true">
				<div id="top">
					<label for="NEW_PLD_TREE" class="label">' . I18N::translate('Family tree') . '</label>
					<select name="NEW_PLD_TREE" id="NEW_PLD_TREE" class="tree">';
		foreach (Tree::getAll() as $tree):
			if ($tree->getTreeId() == $WT_TREE->getTreeId()) {
				$html .= '<option value="' . $tree->getTreeId() . '" data-ged="' . $tree->getName() . '" selected="selected">' . $tree->getTitle() . '</option>';
			} else {
				$html .= '<option value="' . $tree->getTreeId() . '" data-ged="' . $tree->getName() . '">' . $tree->getTitle() . '</option>';
			}
		endforeach;
		
		
		$html .= '	</select>
					
					
					<div class="field">
						<label class="label" style="color:black;">' . I18N::translate('Gallery of pictures exists?') . '</label>' .
							FunctionsEdit::twoStateCheckbox('NEW_PLD_OPTIONS[GALLERY]', 0) . '
					</div>
					
					<br>
					<div class="field" title="' . I18N::translate('Direct access with adding of the name of the city at the end of the url (Look source and readme)') . '">
						<label class="label" style="color:black;">' . I18N::translate('Direct access to the gallery of pictures ?') . '</label>' .
							FunctionsEdit::twoStateCheckbox('NEW_PLD_OPTIONS[GALLERY_LOC]',0) . '
					</div>
					<br>
					
					<div class="field">
						<label for="NEW_PLD_GALLERY_URL" class="label" style="color:black;width:500px;">' . I18N::translate('Add a gallery url')  . '</label>
						<input type="text" id="NEW_PLD_GALLERY_URL" class="gallery" name="NEW_PLD_GALLERY_URL" value="http://www.adumas.1s.fr/pwg" />
						';
				
				$html .= '</div>
					
					<br>
					
					<div class="field">
						<label for="NEW_PLD_DELCAMPE_URL" class="label" style="color:black;width:500px;">' . 
							I18N::translate('Add a delcampe url')  . '</label>
						<input type="text" id="NEW_PLD_DELCAMPE_URL" class="gallery" name="NEW_PLD_DELCAMPE_URL" value="http://www.delcampe.fr/items?language=F&searchString=" />
						</div>
					<br>
					<div class="field">
						<label for="NEW_PLD_CASSINI_URL" class="label" style="color:black;">' . 
						I18N::translate('Add Cassini\'s maps url') .  '</label>
						<input type="text" id="NEW_PLD_CASSINI_URL" class="gallery" name="NEW_PLD_CASSINI_URL" value=" http://rumsey.geogarage.com/maps/cassinige.html" />
					</div>
					
					<br>
					
					<fieldset><legend>' . I18N::translate('Welcome page') . '</legend>
					<div class="field" title="' . I18N::translate('Take care to use >> _<< to séparate the words !!') . '">
						<label for="NEW_PLD_WELCOME_SOURCE" class="label" style="color:black;">' . 
						I18N::translate('Take care to use >> _<< to séparate the words !!') . '<p> S_Ville_Lat_Long</p>' .  '</label>
						<input type="text" id="NEW_PLD_WELCOME_SOURCE" class="gallery" name="NEW_PLD_WELCOME_SOURCE" value="S_Paris_E2.4167_N48.8167" />												
					</div>
					</fieldset>
				</div>';
				
				
		
		$html .='<hr/>
								
				<button id="fsubmit" class="btn btn-primary" type="submit" >						
							<i class="fa fa-check"></i>' . I18N::translate('Save') . '</button>
				
			</form>
			</div>';

	echo $html;
	}

	/////////////////// Header ///////////////////

	private function getBandeau($controller){
		$linkdoc = Site::getPreference('MY_VERSIONS_DOC_BASE_URL') . $this->getName() . '.html';
		$linklog = Site::getPreference('MY_VERSIONS_DOC_BASE_URL') . $this->getName() . '_log.txt';
		$linkupd = 'module.php?mod=' . $this->getName() . '&mod_action=check_version';
		
 		//update version?
		myUpdateFunctions::checkNewModsVersions();
		$spref = Site::getPreference('LATEST_MY_MODS_VERSION');
		
		if ($spref !== null){
			$new_version = unserialize($spref)[$this->getName()]['version'];
		} else {		
			$new_version = self::CUSTOM_VERSION;
		}

		$html = '<ol class="breadcrumb small">
			<li><a href="admin.php">'. I18N::translate('Control panel') . '</a></li>
			<li><a href="admin_modules.php">' . I18N::translate('Module administration') . '</a></li>
			<li class="active">' . $controller->getPageTitle() . '</li>
		</ol>';
		$html .= '<div style="
						border:2px solid black;
						border-radius:3px;
						padding:10px;">';
					
		$html .= '	<table style="width:95%;">
			<tr>
			<td style="width:65%; text-align:center;">
			<h3>' . $this->getTitle() . '  V-' . self::CUSTOM_VERSION . '   ' . '<span style="font-size:60%;">(' . 
							self::CUSTOM_VERSION_DATE . ')</span></h3>
			</td>
			<td style="text-align:center;"><span>
				<a href="' . $linkdoc . '">' .  I18N::translate('Documentation') . '
				</a></span></td>
			<td style="text-align:center;"><span>
			<a href="' . $linklog . '">' .  I18N::translate('Change.log') . '
			</a></span></td>
			<td style="text-align:center;" >
			<a href="' . $linkupd . '" title="' . I18N::translate('Checking is done every week, but you can force it now')  . '"><button 
												style="background-color:lightgrey;"
												>' . I18N::translate('Check new version now!') . '</button></a>
			</td>
			</tr>';
			
		$html .= '<tr><td style="text-align:center;" colspan="3">';
		//new version?
		$html .= myUpdateFunctions::lookForNewVersion($this->getName(), self::CUSTOM_VERSION, $new_version ) ;
		$html .= '</td></tr>';
		
		$html .= '</table></div><br>';

	return $html;
	}
	
	public function forceUpdate(){
		myUpdateFunctions::checkNewModsVersions($now = true);
		header('Location:module.php?mod=' . $this->getName() .'&mod_action=admin_config');
	}	
}
