<?php
// webtrees: Web based Family History software
// Copyright (C) 2014 webtrees development team.
//
// Derived from PhpGedView
// Copyright (C) 2010 John Finlay
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
//-------------------------------------------
// http://www.adumas.1s.fr/wt/   
// André DUMAS  July 2015
// Place_doc 1-7-1-1 for WT 1.7.1  
// Many changes: add a class PlacesDocClass including the functions for Places
/*
* A code is associated with each place  in France
* All the documents of a place (pictures, pdf, text..) 
* are associated  with a source SXXXX named S_code (S_80000 for example) 
*  ====...TEST...====
*/
/*
*  V-1-7-2-1 modif WT_LOCALE  23-11-2015
*  Fix bugs
*/
//============================================
 /*     ..............NEW Version ..............
*     Module PlaceDoc V-2-7-2-1
*       Place_doc V-2-7-2-1
*     André DUMAS DEC 2015
*     pour Webtrees v.1.7.2
*     http://www.adumas.1s.fr/wt
*      http://www.adumas.1s.fr/download/
*
*     .............Voir Place_doc.txt...................
*/

namespace Andre\WebtreesAddOn\PlaceDoc;

use Composer\Autoload\ClassLoader;
use Fisharebest\Webtrees\Controller\BaseController;
use Fisharebest\Webtrees\Controller\PageController;
use Fisharebest\Webtrees\Controller\SourceController;
use Fisharebest\Webtrees\Filter;
use Fisharebest\Webtrees\I18N;
use Fisharebest\Webtrees\Log;
use Fisharebest\Webtrees\Module\AbstractModule;
use Fisharebest\Webtrees\Module\ModuleConfigInterface;
use Fisharebest\Webtrees\Module\ModuleMenuInterface;
use Fisharebest\Webtrees\Module\ModuleTabInterface;
use Fisharebest\Webtrees\Individual;
use Fisharebest\Webtrees\Menu;
use Fisharebest\Webtrees\Media;
use Fisharebest\Webtrees\Auth;
use Fisharebest\Webtrees\Source;
use Fisharebest\Webtrees\Tree;
use Fisharebest\Webtrees\Database;
use Fisharebest\Webtrees\Functions;
use Fisharebest\Webtrees\Functions\FunctionsEdit;


use Andre\WebtreesAddOn\PlaceDoc\Template\AdminTemplate;

/**
 * Class PlaceDocModule
 */

class PlaceDocModule extends AbstractModule implements ModuleConfigInterface, ModuleTabInterface {
		
		CONST CUSTOM_VERSION = '1-7-9-1a';
		CONST CUSTOM_VERSION_DATE = '18-12-2017';
		CONST CUSTOM_VERSION_AUTHOR = 'André DUMAS FRANCE';
		
		// How to update the database schema for this module
	const SCHEMA_TARGET_VERSION			= 1;
	const SCHEMA_SETTING_NAME			= 'PLD_SCHEMA_VERSION';
	const SCHEMA_MIGRATION_PREFIX		= '\Andre\WebtreesAddOn\PlaceDoc\Schema';
			
		
		private $facts;
		private $places = Array();
		private $notes = array();
		protected $lien;

		var $headers; // CSS and script to include in the top of <head> section, before theme’s CSS
		var $js; // the PlaceDocHandler javascript
		
	

	public function __construct() {
		parent::__construct(__DIR__);

		// register the namespaces
		$loader = new ClassLoader();
		$loader->addPsr4('Andre\\WebtreesAddOn\\PlaceDoc\\', WT_MODULES_DIR . $this->getName() . '/app');
		$loader->register();
		
		if ( $this->getSetting('CUSTOM_VERSION') != self::CUSTOM_VERSION){
			$this->setSetting('CUSTOM_VERSION', self::CUSTOM_VERSION); //for my desk
		}
		//Database::updateSchema(self::SCHEMA_MIGRATION_PREFIX, self::SCHEMA_SETTING_NAME, self::SCHEMA_TARGET_VERSION);
	
	
		
	}
		
	
	/** {@inheritdoc} */
	public function getTitle() {
		return /* I18N: Name of a module */ I18N::translate('Place documentation');
	}

	/** {@inheritdoc} */
	public function getDescription() {
		return /* I18N: Description of the “Sources” module */ I18N::translate('A tab displaying documentation about the places from the Facts tab.');
	}

	

	/** {@inheritdoc} */
	public function hasTabContent() {
		return true;
	}

	/** {@inheritdoc} */
	public function isGrayedOut() {
		return !$this->getFactsWithPlaces();
	}


	// Implement WT_Module_Config
	public function getConfigLink() {
		return 'module.php?mod=' . $this->getName() . '&amp;mod_action=admin_config';
	}

		/** {@inheritdoc} */
	public function defaultTabOrder() {
		return 8;
	}
	

	// Reset all settings to default
	private function pld_reset() {
		Database::prepare("DELETE FROM `##module_setting` WHERE setting_name LIKE 'PLD%'")->execute();
		Log::addConfigurationLog($this->getTitle() . ' reset to default values');
	}

	// Delete item
	private function pld_delete() {
		$PLD_SETTINGS = unserialize($this->getSetting('PLD_SETTINGS'));
		unset($PLD_SETTINGS[Filter::getInteger('key')]);
		$NEW_PLD_SETTINGS = array_merge($PLD_SETTINGS);
		$this->setSetting('PLD_SETTINGS', serialize($NEW_PLD_SETTINGS));
		Log::addConfigurationLog($this->getTitle() . ' item deleted');
	}

	// Actions from the configuration page
	private function config() {

	}

	

	/** {@inheritdoc} */
	public function getTabContent() {
		global $controller, $pid, $WT_TREE, $GEDCOM;
		global $TEXT_DIRECTION;
		global $SEARCH_SPIDER;
 
			/* update schema */
			Database::updateSchema(self::SCHEMA_MIGRATION_PREFIX, self::SCHEMA_SETTING_NAME, self::SCHEMA_TARGET_VERSION);
	
	//====================================		

		ob_start();		
		?>

<html>
  <body> 
  
 <script>
					var pid ='<?php echo Filter::get('pid'); ?>' ;					 
					var ad = jQuery(location).attr('href');
					var chemin = WT_SCRIPT_NAME + '?pid=' + pid + '&'+ WT_GEDCOM;
					var chemin1 = 'module.php?mod=' + <?php echo "'". $this->getName()."'"; ?> ;
					var source = <?php echo "'". $this->getSetting("PLD_SETTING_WELCOME_SOURCE")."'"; ?> ;
					
					function openPdfDlg(){
						$('div#pdflink').dialog('open');
					}
</script>
  <script src= "
  <?php echo WT_MODULES_DIR.$this->getName().'/js/my.js'; ?> 
  "></script>
  
  <div class="ct">
<div class="et">
<table style="width:100%;"><tr><td>
	<select id="pays" name="pays" >
		<option value="">Choix du pays</option>
	</select>
</td>
<td class="tdsp"><span><b><i>Documents relatifs aux lieux cités: Choisir le pays puis la ville</i></b></span></td>
<td>
<select id="ville" name="ville">
		<option value="0">Choix de la ville</option>
	</select>
</td></tr>
</table>
</div>

<div class="sp"></div>
<div id="fg"><br></div>
						
<div id="fd">
           <span>Pas de photos disponibles</span>		
</div>

</div>

<div id="pdflink"><b>
     <p></p>
	<p><?php  echo I18N::translate('Document available');?></p>
	<p></p>
	<span><?php  echo I18N::translate('No document available'); ?></span>
</b></div>

<div id="noteslink"><b>
     <p></p>
	<p><?php echo I18N::translate('Add a new note'); ?></p>
	<p></p>
	<textarea col="40" rows="7"></textarea>
</b></div>
<div>



</body></html>
		
<?php
		return '<div id="' . $this->getName() . '_content">' . ob_get_clean() . '</div>';
	}
	
	//======================================
	
	
	
//============================================
//Affichage des images
//param: $imgs 
// Return : string html pour ajax
//---------------------------------------------

public function  AjDisplayMenuPdf($imgs) {
	
	 $htmlpdf = '';
	 $html = '';
		
	foreach ($imgs as $img) :		
		if (($img->extension()) == 'pdf') :	  		
		 $lien = $img->getHtmlUrlDirect(); 
			$html .= ' <li>
							<a href="'.$lien.' ">'
							.basename($img->getFilename()).
							'</a></li> 
							';										
			endif;				  				
	 endforeach;  
	 return $html;
}


public function  AjImgDisplay_fd($imgs) {	
	
	 $htmlpdf = '';
	if (count($this->notes) > 0) { 
		$but_notes = '<button class="listNotes" >' . I18N::translate('Notes') . '</button>';
	} else {
		$but_notes = '';
	}
	 $liste = '';
	
	$html = $but_notes;
	$w = 0; $h = 0;
	
	foreach ($imgs as $img) :			 
		if (($img->extension()) != 'pdf') :
		  $w = mt_rand(7, 600);
		  $h = mt_rand(5, 150) ;
		  $html .= "
					<div class='media-display-image'" .  
							" style=\"left:" . $w . "px; top:" . $h . "px\" >" .
							$img->displayImage() .
					"</div>						  
					";
			endif;				
	endforeach; 
	 return $html;
}


	//=========================================
	// Display menu in #fg
	// voir module config
	//=========================================
		
public  function displayMenufgWT($name, $code) {
	global $WT_TREE;	
	$pathDocs = '';
	
	if ( substr($code, 0, 2) == 'S_' ){$scode = substr($code, 2); } else {$scode = $code;}
	
	if (!($sid = $this->module()->existSource($scode, $WT_TREE->getTreeId()))){
			$sid = $this->module()->existSource(substr($this->getSetting('PLD_SETTING_WELCOME_SOURCE'), 2), $WT_TREE->getTreeId()) ;//default:		
	}
	
	 
	 // On extrait Long et Lat de $code
	 //code de la forme S_Ville_long_lat
	 $coord = $this->get_coord_place($code); 
	    if (empty($coord)){
			 //Paris par defaut S_Paris_E2.4167_N48.8167
			 $coord['long'] = 'E2.4167' ;
			 $coord['lat'] = 'N48.8167';
		}
	 $zoom = 12;  //valeur par défaut pour les cartes de Cassini de ce site
	 $villeName  = $name;
	 
	  $GALLERY_URL  = $this->getSetting("PLD_SETTING_GALLERY_URL");
	  $CASSINI_URL  = $this->getSetting("PLD_SETTING_CASSINI_URL");
	  $DELCAMPE_URL = $this->getSetting("PLD_SETTING_DELCAMPE_URL");
	  
	//http:// indispensable devant les link
	$link_gal = $GALLERY_URL."/index.php?/category/".$this->delAccents($villeName).'"target="_blank" ';
	$link_wpd = 'http://'.WT_LOCALE.'.wikipedia.org/wiki/'.$villeName.'"target="_blank" ';
	$link_delc = $DELCAMPE_URL.$villeName.'"target="_blank" ';
	
	$link_cas = $CASSINI_URL.'?lat='.substr($coord['lat'],1).'&lon='.substr($coord['long'],1).'&zoom='.$zoom.'"target="_blank" ';
	$link_pdf = '';	
	
	if (Auth::isAdmin() || Auth::isEditor($WT_TREE)  ) {	
		$ad = 'source.php?sid=' . $sid . '&ged=' . $WT_TREE->getName();
		$link_note = " 	
			window.open('"  . $ad . "' , '_blank' , edit_window_specs );
		return false;
		";
		
	} else {
		$link_note = '#';
	}
	
	if (Auth::check()){
		$onclick_pdf = 'onclick="openPdfDlg(); return false;"';
	} else {
		$onclick_pdf = 'return false';		
	}
			

	$gal = new Menu(I18N::translate('My own <br>galleery'), $link_gal);
		$gal->addClass('gal', 'gal');
		
	$wpd = new Menu(I18N::translate('Search<br> on<br> Wikipedia') , $link_wpd);
		$wpd->addClass('wpd', 'wpd');
		
	$delc = new Menu(I18N::translate('Search<br> on<br> Delcampe'), $link_delc);
		$delc->addClass('delc', 'delc');
		
	$cas = new Menu("Carte de <br>Cassini ", $link_cas);
		$cas->addClass('cas', 'cas');

	$not = new Menu("Mes notes ", $link_note);
		$not->addClass('nots', 'nots');
		//$not->setAttrs(array('onclick'=>'openDocsDlg();'));

			$html = '<a href="#" ' . $onclick_pdf . '><div class="pdf">' .I18N::translate('Documents <br> pdf') . 
			'</div></a>' .					
						$gal->getMenu().
						$wpd->getMenu() .
						$delc->getMenu() . 
						$cas->getMenu() .					
						'<a href="#"><div class="nots"  
						title="' . I18N::translate('You must be editor to open the source file  and add media/notes etc...') . '"
						onclick="' . $link_note . ' " 
						>' . I18N::translate('Edit <br>source') . '</div></a>					
					';					
	return $html;
}

private function getSourceNotes($code){	
	global $WT_TREE;
	
	if ( substr($code, 0, 2) == 'S_' ){$scode = substr($code, 2); } else {$scode = $code;}
	
	if (!($sid = $this->module()->existSource($scode, $WT_TREE->getTreeId()))){
			$sid = $this->module()->existSource(substr($this->getSetting('PLD_SETTING_WELCOME_SOURCE'), 2), $WT_TREE->getTreeId()) ;//default:		
	}
	
	$record     = Source::getInstance($sid, $WT_TREE);
	if (!null == $record){
	$gs = $record->getFacts('NOTE');
	} else {
		$gs = array();		
	}
	if (count($gs) > 0 ){
		$html = '';
		foreach ($gs as $g){ 
			$html .= 'Note: ' .$g->getValue() . PHP_EOL;
		}
		$html .= '';
		$bouton = '<button class="listNotes" title="' . $html . '">Notes</button>';	
	} else {
		$bouton = '';
	}	
	
	return $bouton;
}
	
	/**
	 * Supprime les accents d'une str
	 * return str sans accents
	 */
	public function delAccents($str) {
 return str_replace(    array( 
                                'À', 'Â', 'Ä', 'Á', 'Ã', 'Å', 
                                'Î', 'Ï', 'Ì', 'Í',  
                                'Ô', 'Ö', 'Ò', 'Ó', 'Õ', 'Ø',  
                                'Ù', 'Û', 'Ü', 'Ú',  
                                'É', 'È', 'Ê', 'Ë',  
                                'Ç', 'Ÿ', 'Ñ', 'Ý',
								'é','è','ê','ë',
								'ü','ö','ö',
								' ',
                            ), 
                            array( 
                                'A', 'A', 'A', 'A', 'A', 'A',  
                                'I', 'I', 'I', 'I',  
                                'O', 'O', 'O', 'O', 'O', 'O',  
                                'U', 'U', 'U', 'U',  
                                'E', 'E', 'E', 'E',  
                                'C', 'Y', 'N', 'Y', 
								'e','e','e','e',
								'u','o','o',
								'-',
								
                            ), 
                            $str
                        ); 
						} 


// Add error or succes message
	private function addMessage($controller, $type, $msg) {
		if ($type == "success") {
			$class = "ui-state-highlight";
		}
		if ($type == "error") {
			$class = "ui-state-error";
		}
		$controller->addInlineJavaScript('
			jQuery("#error").text("' . $msg . '").addClass("' . $class . '").show("normal");
			setTimeout(function() {
				jQuery("#error").hide("normal");
			}, 10000);
		');
	}

	private function module(){
		return new PlacesDocClass;
	}
	
	private function template(){
		return new AdminTemplate;
	}

	//=========================================
	public function modAction($mod_action) {
		global $controller, $WT_TREE;
		
		switch ($mod_action) {
		case 'admin_config':
					/* update schema */
		Database::updateSchema(self::SCHEMA_MIGRATION_PREFIX, self::SCHEMA_SETTING_NAME, self::SCHEMA_TARGET_VERSION);
	
				$this->template()->pageContent();
				break;
		case 'menusVides' :
		    Zend_Session::writeClose();			
				echo $this->displayMenu();			
			break;
		
		case 'pays':						
			//header('Content-Type: text/html; charset=UTF-8');
			$pid = Filter::get('pid', WT_REGEX_XREF);			
			$individual = Individual::getInstance($pid, $WT_TREE);
			if ($individual) {				
				echo  $this->module()->genHtmlPays($individual);
			}
			
			break;
		case 'villes':							
				//header('Content-Type: text/html; charset=UTF-8');
			$pid = Filter::get('pid', WT_REGEX_XREF);
			$pays = Filter::get('pays', WT_REGEX_XREF);			
			
			$individual = Individual::getInstance($pid, $WT_TREE);
			if ($individual) {
				echo $this->module()->genHtmlVilles($individual, $pays);
			}
			
			break;
			
		case 'source' :
			$but =  $this->getSourceNotes(Filter::post('code'));
		    $source = Filter::post('source');
			$images = $this->getIdMedia($source, $WT_TREE->getTreeId());
			if ($images != null) {
				foreach ($images as $image) {   
					$imgs[] = Media::getInstance($image->l_to, $WT_TREE);
				}
				echo $but . $this->AjImgDisplay_fd($imgs);
			}  else {
				echo $but;
			} 
			break;
		
		case 'menupdf' :		    
			$source = Filter::post('source');
			$images = $this->getIdMedia($source, $WT_TREE->getTreeId());
			if ($images != null) {
				foreach ($images as $image) {   
					$imgs[] = Media::getInstance($image->l_to, $WT_TREE);
				}
				echo $this->AjDisplayMenuPdf($imgs);
			}  else {
				echo '';
			} 
			break;	
		
		case 'menus' :		    
			$name = Filter::post('name');
			$code = Filter::post('code');
			
			if (($name != null) && ($name != '')) {
				//genere le contenu html de #fg
				echo $this->displayMenufgWT($name, $code);
			}  else {
				echo '';
			} 
			break;
			case 'addnotes' :
				$source = substr(Filter::post('source'), 2);
				$note = Filter::post('note');
				if ( ($sid = $this->module()->existSource($source, $WT_TREE->getTreeId())) !== null ){
					
				}
			break;
		default:
		
			header('HTTP/1.0 404 Not Found');
			break;
		}
	}

	/**
	 * get lat & long from code  
	 *  code de la forme S_Ville_long_lat
	 */
	private function get_coord_place($code)  {
		
	//code de la forme S_Ville_long_lat
	$tmp = explode('_', $code);
	
	if (count($tmp) >= 2) {
	$table = array(
					'lat' => $tmp[count($tmp)-1], 
					'long' => $tmp[count($tmp)-2]
					);		
	} else {$table = array();}				
					
			return $table;
			
	}	
	
	
	
	/**
	 * Get all the facts for an individual which contain sources.
	 *
	 * @return WT_Fact[]
	 */
	private function getFactsWithSources() {
		global $controller;

		if ($this->facts === null) {
			$facts = $controller->record->getFacts();
			foreach ($controller->record->getSpouseFamilies() as $family) {
				if ($family->canShow()) {
					foreach ($family->getFacts() as $fact) {
						$facts[] = $fact;
					}
				}
			}
			$this->facts = array();
			foreach ($facts as $fact) {
				if (preg_match('/(?:^1|\n\d) SOUR/', $fact->getGedcom())) {
					$this->facts[] = $fact;
				}
			}
			sort_facts($this->facts);
		}

		return $this->facts;
	}

	
	//=============================
	public function canLoadAjax() {
		return true;
	}


	
		//================================
	//Trouver les media associés à la source S_INSEE->SXXXX
	//pour le gedcom courant
	// return MYYYY
	//l_file = gid
	//l_from  SXXXX
	//l_type OBJE
	//l_to MYYYY
	//================================
private function getIdMedia($source, $gid ) {
	
	$req = Database::prepare ('
	      SELECT SQL_CACHE l_to FROM ##link AS l
		  LEFT JOIN ##sources AS s ON  s.s_id = l.l_from
		     WHERE  s.s_name = ?
			 AND l.l_file = ?
			 AND l.l_type = "OBJE"
				');		
	   return $req->execute(array($source, $gid))->fetchAll();
	
}	
	//================================
	
	
	/** {@inheritdoc} */
	public function getPreLoadContent() {
		return 
		'<script>
			if (document.createStyleSheet) {
				document.createStyleSheet("' . $this->css() . '"); // For Internet Explorer
			} else {
				var newSheet=document.createElement("link");
    		newSheet.setAttribute("rel","stylesheet");
    		newSheet.setAttribute("type","text/css");
   			newSheet.setAttribute("href","' . $this->css() . '");
		    document.getElementsByTagName("head")[0].appendChild(newSheet);
			}
			</script>
		';
	}

	/**
	 * @return string
	 */
	public function css() {
		return WT_STATIC_URL . WT_MODULES_DIR . $this->getName() . '/css/my.css';
	}

	/**
	 * @return string
	 */
	public function js() {
		return WT_STATIC_URL . WT_MODULES_DIR . $this->getName() . '/js/';
	}
	
	private function getFactsWithPlaces() {
		return true;
		
	}
	

	

	
	}

	return new PlaceDocModule;